@@ -6,6 +6,7 @@ class FluxUtils:
     @staticmethod
     def default_error_resume(value, ex, context):
         raise value
+
     @staticmethod
     def default_action(value, context):
         pass
@@ -24,9 +25,13 @@ class FluxUtils:
         raise e
 
     @staticmethod
-    def try_or(statement, value, ctx) -> (object, Exception):
+    def default_finish(context):
+        pass
+
+    @staticmethod
+    def try_or(statement, *args) -> (object, Exception):
         try:
-            v = statement(value, ctx)
+            v = statement(*args)
             return v, None
         except Exception as ex:
             return None, ex
